@@ -18,52 +18,61 @@
 package cache
 
 import (
-	"time"
+	"sync/atomic"
 
-	"codeberg.org/gruf/go-cache/v3/simple"
-	"codeberg.org/gruf/go-cache/v3/ttl"
+	"code.superseriousbusiness.org/gotosocial/internal/cache/domain"
+	"code.superseriousbusiness.org/gotosocial/internal/config"
+	"code.superseriousbusiness.org/gotosocial/internal/gtsmodel"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
 	"codeberg.org/gruf/go-structr"
-	"github.com/superseriousbusiness/gotosocial/internal/cache/domain"
-	"github.com/superseriousbusiness/gotosocial/internal/config"
-	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
 )
 
-type GTSCaches struct {
+type DBCaches struct {
 	// Account provides access to the gtsmodel Account database cache.
 	Account StructCache[*gtsmodel.Account]
 
 	// AccountNote provides access to the gtsmodel Note database cache.
 	AccountNote StructCache[*gtsmodel.AccountNote]
 
-	// TEMPORARY CACHE TO ALLEVIATE SLOW COUNT QUERIES,
-	// (in time will be removed when these IDs are cached).
-	AccountCounts *simple.Cache[string, struct {
-		Statuses int
-		Pinned   int
-	}]
-
 	// AccountSettings provides access to the gtsmodel AccountSettings database cache.
 	AccountSettings StructCache[*gtsmodel.AccountSettings]
 
+	// AccountStats provides access to the gtsmodel AccountStats database cache.
+	AccountStats StructCache[*gtsmodel.AccountStats]
+
 	// Application provides access to the gtsmodel Application database cache.
 	Application StructCache[*gtsmodel.Application]
 
 	// Block provides access to the gtsmodel Block (account) database cache.
 	Block StructCache[*gtsmodel.Block]
 
-	// FollowIDs provides access to the block IDs database cache.
+	// BlockIDs provides access to the block IDs database cache.
 	BlockIDs SliceCache[string]
 
 	// BoostOfIDs provides access to the boost of IDs list database cache.
 	BoostOfIDs SliceCache[string]
 
+	// Conversation provides access to the gtsmodel Conversation database cache.
+	Conversation StructCache[*gtsmodel.Conversation]
+
+	// ConversationLastStatusIDs provides access to the conversation last status IDs database cache.
+	ConversationLastStatusIDs SliceCache[string]
+
 	// DomainAllow provides access to the domain allow database cache.
 	DomainAllow *domain.Cache
 
 	// DomainBlock provides access to the domain block database cache.
 	DomainBlock *domain.Cache
 
+	// DomainPermissionDraft provides access to the domain permission draft database cache.
+	DomainPermissionDraft StructCache[*gtsmodel.DomainPermissionDraft]
+
+	// DomainPermissionSubscription provides access to the domain permission subscription database cache.
+	DomainPermissionSubscription StructCache[*gtsmodel.DomainPermissionSubscription]
+
+	// DomainPermissionExclude provides access to the domain permission exclude database cache.
+	DomainPermissionExclude *domain.Cache
+
 	// Emoji provides access to the gtsmodel Emoji database cache.
 	Emoji StructCache[*gtsmodel.Emoji]
 
@@ -73,6 +82,10 @@ type GTSCaches struct {
 	// Filter provides access to the gtsmodel Filter database cache.
 	Filter StructCache[*gtsmodel.Filter]
 
+	// FilterIDs provides access to the filter IDs database cache.
+	// This cache is keyed as: {accountID} -> []{filterIDs}.
+	FilterIDs SliceCache[string]
+
 	// FilterKeyword provides access to the gtsmodel FilterKeyword database cache.
 	FilterKeyword StructCache[*gtsmodel.FilterKeyword]
 
@@ -84,10 +97,23 @@ type GTSCaches struct {
 
 	// FollowIDs provides access to the follower / following IDs database cache.
 	// THIS CACHE IS KEYED AS THE FOLLOWING {prefix}{accountID} WHERE PREFIX IS:
-	// - '>'  for following IDs
-	// - 'l>' for local following IDs
-	// - '<'  for follower IDs
-	// - 'l<' for local follower IDs
+	//
+	// - '>{$accountID}'  for following IDs
+	//   e.g. FollowIDs.Load(">" + account.ID, func() {})
+	//   which will load a slice of follows IDs FROM account.
+	//
+	// - 'l>{$accountID}' for local following IDs
+	//   e.g. FollowIDs.Load("l>" + account.ID, func() {})
+	//   which will load a slice of LOCAL follows IDs FROM account.
+	//
+	// - '<{$accountID}' for follower IDs
+	//   e.g. FollowIDs.Load("<" + account.ID, func() {})
+	//   which will load a slice of follows IDs TARGETTING account.
+	//
+	// - 'l<{$accountID}' for local follower IDs
+	//   e.g. FollowIDs.Load("l<" + account.ID, func() {})
+	//   which will load a slice of LOCAL follows IDs TARGETTING account.
+	//
 	FollowIDs SliceCache[string]
 
 	// FollowRequest provides access to the gtsmodel FollowRequest database cache.
@@ -95,21 +121,76 @@ type GTSCaches struct {
 
 	// FollowRequestIDs provides access to the follow requester / requesting IDs database
 	// cache. THIS CACHE IS KEYED AS THE FOLLOWING {prefix}{accountID} WHERE PREFIX IS:
-	// - '>'  for following IDs
-	// - '<'  for follower IDs
+	//
+	// - '>{$accountID}'  for follow request IDs
+	//   e.g. FollowRequestIDs.Load(">" + account.ID, func() {})
+	//   which will load a slice of follow request IDs TARGETTING account.
+	//
+	// - '<{$accountID}'  for follow request IDs
+	//   e.g. FollowRequestIDs.Load("<" + account.ID, func() {})
+	//   which will load a slice of follow request IDs FROM account.
+	//
 	FollowRequestIDs SliceCache[string]
 
+	// FollowingTagIDs provides access to account IDs following / tag IDs followed by
+	// account db cache. THIS CACHE IS KEYED AS THE FOLLOWING {prefix}{id} WHERE:
+	//
+	// - '>{$accountID}' for tag IDs followed by account
+	//   e.g. FollowingTagIDs.Load(">" + account.ID, func() {})
+	//   which will load a slice of tag IDs followed by account.
+	//
+	// - '<{$tagIDs}' for account IDs following tag
+	//   e.g. FollowingTagIDs.Load("<" + tag.ID, func() {})
+	//   which will load a slice of account IDs following tag.
+	//
+	FollowingTagIDs SliceCache[string]
+
 	// Instance provides access to the gtsmodel Instance database cache.
 	Instance StructCache[*gtsmodel.Instance]
 
+	// LocalInstance provides caching for
+	// simple + common local instance queries.
+	LocalInstance struct {
+		Domains  atomic.Pointer[int]
+		Statuses atomic.Pointer[int]
+		Users    atomic.Pointer[int]
+		UserIDs  atomic.Pointer[[]string]
+	}
+
+	// InteractionRequest provides access to the gtsmodel InteractionRequest database cache.
+	InteractionRequest StructCache[*gtsmodel.InteractionRequest]
+
 	// InReplyToIDs provides access to the status in reply to IDs list database cache.
 	InReplyToIDs SliceCache[string]
 
 	// List provides access to the gtsmodel List database cache.
 	List StructCache[*gtsmodel.List]
 
-	// ListEntry provides access to the gtsmodel ListEntry database cache.
-	ListEntry StructCache[*gtsmodel.ListEntry]
+	// ListIDs provides access to the list IDs owned by account / list IDs follow
+	// contained in db cache. THIS CACHE IS KEYED AS FOLLOWING {prefix}{id} WHERE:
+	//
+	// - 'a{$accountID}' for list IDs owned by account
+	//   e.g. ListIDs.Load("a" + account.ID, func() {})
+	//   which will load a slice of list IDs owned by account.
+	//
+	// - 'f{$followID}' for list IDs follow contained in
+	//   e.g. ListIDs.Load("f" + follow.ID, func() {})
+	//   which will load a slice of list IDs containing follow.
+	//
+	ListIDs SliceCache[string]
+
+	// ListedIDs provides access to the account IDs in list / follow IDs in
+	// list db cache. THIS CACHE IS KEYED AS FOLLOWING {prefix}{id} WHERE:
+	//
+	// - 'a{listID}' for account IDs in list ID
+	//   e.g. ListedIDs.Load("a" + list.ID, func() {})
+	//   which will load a slice of account IDs in list.
+	//
+	// - 'f{listID}' for follow IDs in list ID
+	//   e.g. ListedIDs.Load("f" + list.ID, func() {})
+	//   which will load a slice of follow IDs in list.
+	//
+	ListedIDs SliceCache[string]
 
 	// Marker provides access to the gtsmodel Marker database cache.
 	Marker StructCache[*gtsmodel.Marker]
@@ -138,9 +219,24 @@ type GTSCaches struct {
 	// Report provides access to the gtsmodel Report database cache.
 	Report StructCache[*gtsmodel.Report]
 
+	// ScheduledStatus provides access to the gtsmodel ScheduledStatus database cache.
+	ScheduledStatus StructCache[*gtsmodel.ScheduledStatus]
+
+	// SinBinStatus provides access to the gtsmodel SinBinStatus database cache.
+	SinBinStatus StructCache[*gtsmodel.SinBinStatus]
+
 	// Status provides access to the gtsmodel Status database cache.
 	Status StructCache[*gtsmodel.Status]
 
+	// StatusBookmark provides access to the gtsmodel StatusBookmark database cache.
+	StatusBookmark StructCache[*gtsmodel.StatusBookmark]
+
+	// StatusBookmarkIDs provides access to the status bookmark IDs list database cache.
+	StatusBookmarkIDs SliceCache[string]
+
+	// StatusEdit provides access to the gtsmodel StatusEdit database cache.
+	StatusEdit StructCache[*gtsmodel.StatusEdit]
+
 	// StatusFave provides access to the gtsmodel StatusFave database cache.
 	StatusFave StructCache[*gtsmodel.StatusFave]
 
@@ -150,18 +246,32 @@ type GTSCaches struct {
 	// Tag provides access to the gtsmodel Tag database cache.
 	Tag StructCache[*gtsmodel.Tag]
 
-	// Tombstone provides access to the gtsmodel Tombstone database cache.
-	Tombstone StructCache[*gtsmodel.Tombstone]
-
 	// ThreadMute provides access to the gtsmodel ThreadMute database cache.
 	ThreadMute StructCache[*gtsmodel.ThreadMute]
 
+	// Token provides access to the gtsmodel Token database cache.
+	Token StructCache[*gtsmodel.Token]
+
+	// Tombstone provides access to the gtsmodel Tombstone database cache.
+	Tombstone StructCache[*gtsmodel.Tombstone]
+
 	// User provides access to the gtsmodel User database cache.
 	User StructCache[*gtsmodel.User]
 
-	// Webfinger provides access to the webfinger URL cache.
-	// TODO: move out of GTS caches since unrelated to DB.
-	Webfinger *ttl.Cache[string, string] // TTL=24hr, sweep=5min
+	// UserMute provides access to the gtsmodel UserMute database cache.
+	UserMute StructCache[*gtsmodel.UserMute]
+
+	// UserMuteIDs provides access to the user mute IDs database cache.
+	UserMuteIDs SliceCache[string]
+
+	// VAPIDKeyPair caches the server's VAPID key pair.
+	VAPIDKeyPair atomic.Pointer[gtsmodel.VAPIDKeyPair]
+
+	// WebPushSubscription provides access to the gtsmodel WebPushSubscription database cache.
+	WebPushSubscription StructCache[*gtsmodel.WebPushSubscription]
+
+	// WebPushSubscriptionIDs provides access to the Web Push subscription IDs database cache.
+	WebPushSubscriptionIDs SliceCache[string]
 }
 
 // NOTE:
@@ -193,22 +303,19 @@ func (c *Caches) initAccount() {
 		a2.Emojis = nil
 		a2.AlsoKnownAs = nil
 		a2.Move = nil
+		a2.MovedTo = nil
 		a2.Settings = nil
+		a2.Stats = nil
 
 		return a2
 	}
 
-	c.GTS.Account.Init(structr.CacheConfig[*gtsmodel.Account]{
+	c.DB.Account.Init(structr.CacheConfig[*gtsmodel.Account]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
-			{Fields: "URL"},
-			{Fields: "Username,Domain", AllowZero: true},
 			{Fields: "PublicKeyURI"},
-			{Fields: "InboxURI"},
-			{Fields: "OutboxURI"},
-			{Fields: "FollowersURI"},
-			{Fields: "FollowingURI"},
+			{Fields: "Username,Domain", AllowZero: true},
 		},
 		MaxSize:    cap,
 		IgnoreErr:  ignoreErrors,
@@ -217,22 +324,6 @@ func (c *Caches) initAccount() {
 	})
 }
 
-func (c *Caches) initAccountCounts() {
-	// Simply use size of accounts cache,
-	// as this cache will be very small.
-	cap := c.GTS.Account.Cap()
-	if cap == 0 {
-		panic("must be initialized before accounts")
-	}
-
-	log.Infof(nil, "cache size = %d", cap)
-
-	c.GTS.AccountCounts = simple.New[string, struct {
-		Statuses int
-		Pinned   int
-	}](0, cap)
-}
-
 func (c *Caches) initAccountNote() {
 	// Calculate maximum cache size.
 	cap := calculateResultCacheMax(
@@ -255,7 +346,7 @@ func (c *Caches) initAccountNote() {
 		return n2
 	}
 
-	c.GTS.AccountNote.Init(structr.CacheConfig[*gtsmodel.AccountNote]{
+	c.DB.AccountNote.Init(structr.CacheConfig[*gtsmodel.AccountNote]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "AccountID,TargetAccountID"},
@@ -275,7 +366,7 @@ func (c *Caches) initAccountSettings() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.AccountSettings.Init(structr.CacheConfig[*gtsmodel.AccountSettings]{
+	c.DB.AccountSettings.Init(structr.CacheConfig[*gtsmodel.AccountSettings]{
 		Indices: []structr.IndexConfig{
 			{Fields: "AccountID"},
 		},
@@ -289,6 +380,29 @@ func (c *Caches) initAccountSettings() {
 	})
 }
 
+func (c *Caches) initAccountStats() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofAccountStats(), // model in-mem size.
+		config.GetCacheAccountStatsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.AccountStats.Init(structr.CacheConfig[*gtsmodel.AccountStats]{
+		Indices: []structr.IndexConfig{
+			{Fields: "AccountID"},
+		},
+		MaxSize:   cap,
+		IgnoreErr: ignoreErrors,
+		Copy: func(s1 *gtsmodel.AccountStats) *gtsmodel.AccountStats {
+			s2 := new(gtsmodel.AccountStats)
+			*s2 = *s1
+			return s2
+		},
+	})
+}
+
 func (c *Caches) initApplication() {
 	// Calculate maximum cache size.
 	cap := calculateResultCacheMax(
@@ -304,14 +418,15 @@ func (c *Caches) initApplication() {
 		return a2
 	}
 
-	c.GTS.Application.Init(structr.CacheConfig[*gtsmodel.Application]{
+	c.DB.Application.Init(structr.CacheConfig[*gtsmodel.Application]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "ClientID"},
 		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateApplication,
 	})
 }
 
@@ -337,7 +452,7 @@ func (c *Caches) initBlock() {
 		return b2
 	}
 
-	c.GTS.Block.Init(structr.CacheConfig[*gtsmodel.Block]{
+	c.DB.Block.Init(structr.CacheConfig[*gtsmodel.Block]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -360,7 +475,7 @@ func (c *Caches) initBlockIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.BlockIDs.Init(0, cap)
+	c.DB.BlockIDs.Init(0, cap)
 }
 
 func (c *Caches) initBoostOfIDs() {
@@ -371,15 +486,128 @@ func (c *Caches) initBoostOfIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.BoostOfIDs.Init(0, cap)
+	c.DB.BoostOfIDs.Init(0, cap)
+}
+
+func (c *Caches) initConversation() {
+	cap := calculateResultCacheMax(
+		sizeofConversation(), // model in-mem size.
+		config.GetCacheConversationMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(c1 *gtsmodel.Conversation) *gtsmodel.Conversation {
+		c2 := new(gtsmodel.Conversation)
+		*c2 = *c1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		// See internal/db/bundb/conversation.go.
+		c2.Account = nil
+		c2.OtherAccounts = nil
+		c2.LastStatus = nil
+
+		return c2
+	}
+
+	c.DB.Conversation.Init(structr.CacheConfig[*gtsmodel.Conversation]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "ThreadID,AccountID,OtherAccountsKey"},
+			{Fields: "AccountID,LastStatusID"},
+			{Fields: "AccountID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateConversation,
+	})
+}
+
+func (c *Caches) initConversationLastStatusIDs() {
+	cap := calculateSliceCacheMax(
+		config.GetCacheConversationLastStatusIDsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.ConversationLastStatusIDs.Init(0, cap)
 }
 
 func (c *Caches) initDomainAllow() {
-	c.GTS.DomainAllow = new(domain.Cache)
+	c.DB.DomainAllow = new(domain.Cache)
 }
 
 func (c *Caches) initDomainBlock() {
-	c.GTS.DomainBlock = new(domain.Cache)
+	c.DB.DomainBlock = new(domain.Cache)
+}
+
+func (c *Caches) initDomainPermissionDraft() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofDomainPermissionDraft(), // model in-mem size.
+		config.GetCacheDomainPermissionDraftMemRation(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(d1 *gtsmodel.DomainPermissionDraft) *gtsmodel.DomainPermissionDraft {
+		d2 := new(gtsmodel.DomainPermissionDraft)
+		*d2 = *d1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		d2.CreatedByAccount = nil
+
+		return d2
+	}
+
+	c.DB.DomainPermissionDraft.Init(structr.CacheConfig[*gtsmodel.DomainPermissionDraft]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "Domain", Multiple: true},
+			{Fields: "SubscriptionID", Multiple: true},
+		},
+		MaxSize:   cap,
+		IgnoreErr: ignoreErrors,
+		Copy:      copyF,
+	})
+}
+
+func (c *Caches) initDomainPermissionSubscription() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofDomainPermissionSubscription(), // model in-mem size.
+		config.GetCacheDomainPermissionSubscriptionMemRation(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(d1 *gtsmodel.DomainPermissionSubscription) *gtsmodel.DomainPermissionSubscription {
+		d2 := new(gtsmodel.DomainPermissionSubscription)
+		*d2 = *d1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		d2.CreatedByAccount = nil
+
+		return d2
+	}
+
+	c.DB.DomainPermissionSubscription.Init(structr.CacheConfig[*gtsmodel.DomainPermissionSubscription]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "URI"},
+		},
+		MaxSize:   cap,
+		IgnoreErr: ignoreErrors,
+		Copy:      copyF,
+	})
+}
+
+func (c *Caches) initDomainPermissionExclude() {
+	c.DB.DomainPermissionExclude = new(domain.Cache)
 }
 
 func (c *Caches) initEmoji() {
@@ -403,7 +631,7 @@ func (c *Caches) initEmoji() {
 		return e2
 	}
 
-	c.GTS.Emoji.Init(structr.CacheConfig[*gtsmodel.Emoji]{
+	c.DB.Emoji.Init(structr.CacheConfig[*gtsmodel.Emoji]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -432,7 +660,7 @@ func (c *Caches) initEmojiCategory() {
 		return c2
 	}
 
-	c.GTS.EmojiCategory.Init(structr.CacheConfig[*gtsmodel.EmojiCategory]{
+	c.DB.EmojiCategory.Init(structr.CacheConfig[*gtsmodel.EmojiCategory]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "Name"},
@@ -466,17 +694,28 @@ func (c *Caches) initFilter() {
 		return filter2
 	}
 
-	c.GTS.Filter.Init(structr.CacheConfig[*gtsmodel.Filter]{
+	c.DB.Filter.Init(structr.CacheConfig[*gtsmodel.Filter]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "AccountID", Multiple: true},
 		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateFilter,
 	})
 }
 
+func (c *Caches) initFilterIDs() {
+	cap := calculateSliceCacheMax(
+		config.GetCacheFilterIDsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.FilterIDs.Init(0, cap)
+}
+
 func (c *Caches) initFilterKeyword() {
 	// Calculate maximum cache size.
 	cap := calculateResultCacheMax(
@@ -490,23 +729,23 @@ func (c *Caches) initFilterKeyword() {
 		filterKeyword2 := new(gtsmodel.FilterKeyword)
 		*filterKeyword2 = *filterKeyword1
 
-		// Don't include ptr fields that
-		// will be populated separately.
-		// See internal/db/bundb/filter.go.
-		filterKeyword2.Filter = nil
+		// We specifically DO NOT unset
+		// the regexp field here, as any
+		// regexp.Regexp instance is safe
+		// for concurrent access.
 
 		return filterKeyword2
 	}
 
-	c.GTS.FilterKeyword.Init(structr.CacheConfig[*gtsmodel.FilterKeyword]{
+	c.DB.FilterKeyword.Init(structr.CacheConfig[*gtsmodel.FilterKeyword]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
-			{Fields: "AccountID", Multiple: true},
 			{Fields: "FilterID", Multiple: true},
 		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateFilterKeyword,
 	})
 }
 
@@ -522,24 +761,18 @@ func (c *Caches) initFilterStatus() {
 	copyF := func(filterStatus1 *gtsmodel.FilterStatus) *gtsmodel.FilterStatus {
 		filterStatus2 := new(gtsmodel.FilterStatus)
 		*filterStatus2 = *filterStatus1
-
-		// Don't include ptr fields that
-		// will be populated separately.
-		// See internal/db/bundb/filter.go.
-		filterStatus2.Filter = nil
-
 		return filterStatus2
 	}
 
-	c.GTS.FilterStatus.Init(structr.CacheConfig[*gtsmodel.FilterStatus]{
+	c.DB.FilterStatus.Init(structr.CacheConfig[*gtsmodel.FilterStatus]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
-			{Fields: "AccountID", Multiple: true},
 			{Fields: "FilterID", Multiple: true},
 		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateFilterStatus,
 	})
 }
 
@@ -565,7 +798,7 @@ func (c *Caches) initFollow() {
 		return f2
 	}
 
-	c.GTS.Follow.Init(structr.CacheConfig[*gtsmodel.Follow]{
+	c.DB.Follow.Init(structr.CacheConfig[*gtsmodel.Follow]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -588,7 +821,7 @@ func (c *Caches) initFollowIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.FollowIDs.Init(0, cap)
+	c.DB.FollowIDs.Init(0, cap)
 }
 
 func (c *Caches) initFollowRequest() {
@@ -613,7 +846,7 @@ func (c *Caches) initFollowRequest() {
 		return f2
 	}
 
-	c.GTS.FollowRequest.Init(structr.CacheConfig[*gtsmodel.FollowRequest]{
+	c.DB.FollowRequest.Init(structr.CacheConfig[*gtsmodel.FollowRequest]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -636,7 +869,18 @@ func (c *Caches) initFollowRequestIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.FollowRequestIDs.Init(0, cap)
+	c.DB.FollowRequestIDs.Init(0, cap)
+}
+
+func (c *Caches) initFollowingTagIDs() {
+	// Calculate maximum cache size.
+	cap := calculateSliceCacheMax(
+		config.GetCacheFollowingTagIDsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.FollowingTagIDs.Init(0, cap)
 }
 
 func (c *Caches) initInReplyToIDs() {
@@ -647,7 +891,7 @@ func (c *Caches) initInReplyToIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.InReplyToIDs.Init(0, cap)
+	c.DB.InReplyToIDs.Init(0, cap)
 }
 
 func (c *Caches) initInstance() {
@@ -669,14 +913,55 @@ func (c *Caches) initInstance() {
 		i2.DomainBlock = nil
 		i2.ContactAccount = nil
 
-		return i1
+		return i2
 	}
 
-	c.GTS.Instance.Init(structr.CacheConfig[*gtsmodel.Instance]{
+	c.DB.Instance.Init(structr.CacheConfig[*gtsmodel.Instance]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "Domain"},
 		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateInstance,
+	})
+}
+
+func (c *Caches) initInteractionRequest() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofInteractionRequest(),
+		config.GetCacheInteractionRequestMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(i1 *gtsmodel.InteractionRequest) *gtsmodel.InteractionRequest {
+		i2 := new(gtsmodel.InteractionRequest)
+		*i2 = *i1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		// See internal/db/bundb/interaction.go.
+		i2.TargetStatus = nil
+		i2.TargetAccount = nil
+		i2.InteractingAccount = nil
+		i2.Like = nil
+		i2.Reply = nil
+		i2.Announce = nil
+
+		return i2
+	}
+
+	c.DB.InteractionRequest.Init(structr.CacheConfig[*gtsmodel.InteractionRequest]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "InteractionURI"},
+			{Fields: "InteractionRequestURI"},
+			{Fields: "ResponseURI"},
+			{Fields: "AuthorizationURI"},
+		},
 		MaxSize:   cap,
 		IgnoreErr: ignoreErrors,
 		Copy:      copyF,
@@ -700,12 +985,11 @@ func (c *Caches) initList() {
 		// will be populated separately.
 		// See internal/db/bundb/list.go.
 		l2.Account = nil
-		l2.ListEntries = nil
 
 		return l2
 	}
 
-	c.GTS.List.Init(structr.CacheConfig[*gtsmodel.List]{
+	c.DB.List.Init(structr.CacheConfig[*gtsmodel.List]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 		},
@@ -716,37 +1000,26 @@ func (c *Caches) initList() {
 	})
 }
 
-func (c *Caches) initListEntry() {
+func (c *Caches) initListIDs() {
 	// Calculate maximum cache size.
-	cap := calculateResultCacheMax(
-		sizeofListEntry(), // model in-mem size.
-		config.GetCacheListEntryMemRatio(),
+	cap := calculateSliceCacheMax(
+		config.GetCacheListIDsMemRatio(),
 	)
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	copyF := func(l1 *gtsmodel.ListEntry) *gtsmodel.ListEntry {
-		l2 := new(gtsmodel.ListEntry)
-		*l2 = *l1
+	c.DB.ListIDs.Init(0, cap)
+}
 
-		// Don't include ptr fields that
-		// will be populated separately.
-		// See internal/db/bundb/list.go.
-		l2.Follow = nil
+func (c *Caches) initListedIDs() {
+	// Calculate maximum cache size.
+	cap := calculateSliceCacheMax(
+		config.GetCacheListedIDsMemRatio(),
+	)
 
-		return l2
-	}
+	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.ListEntry.Init(structr.CacheConfig[*gtsmodel.ListEntry]{
-		Indices: []structr.IndexConfig{
-			{Fields: "ID"},
-			{Fields: "ListID", Multiple: true},
-			{Fields: "FollowID", Multiple: true},
-		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
-	})
+	c.DB.ListedIDs.Init(0, cap)
 }
 
 func (c *Caches) initMarker() {
@@ -764,7 +1037,7 @@ func (c *Caches) initMarker() {
 		return m2
 	}
 
-	c.GTS.Marker.Init(structr.CacheConfig[*gtsmodel.Marker]{
+	c.DB.Marker.Init(structr.CacheConfig[*gtsmodel.Marker]{
 		Indices: []structr.IndexConfig{
 			{Fields: "AccountID,Name"},
 		},
@@ -789,7 +1062,7 @@ func (c *Caches) initMedia() {
 		return m2
 	}
 
-	c.GTS.Media.Init(structr.CacheConfig[*gtsmodel.MediaAttachment]{
+	c.DB.Media.Init(structr.CacheConfig[*gtsmodel.MediaAttachment]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 		},
@@ -820,10 +1093,16 @@ func (c *Caches) initMention() {
 		m2.OriginAccount = nil
 		m2.TargetAccount = nil
 
+		// Zero non-db fields.
+		m2.NameString = ""
+		m2.IsNew = false
+		m2.TargetAccountURI = ""
+		m2.TargetAccountURL = ""
+
 		return m2
 	}
 
-	c.GTS.Mention.Init(structr.CacheConfig[*gtsmodel.Mention]{
+	c.DB.Mention.Init(structr.CacheConfig[*gtsmodel.Mention]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 		},
@@ -842,7 +1121,7 @@ func (c *Caches) initMove() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.Move.Init(structr.CacheConfig[*gtsmodel.Move]{
+	c.DB.Move.Init(structr.CacheConfig[*gtsmodel.Move]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -883,10 +1162,10 @@ func (c *Caches) initNotification() {
 		return n2
 	}
 
-	c.GTS.Notification.Init(structr.CacheConfig[*gtsmodel.Notification]{
+	c.DB.Notification.Init(structr.CacheConfig[*gtsmodel.Notification]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
-			{Fields: "NotificationType,TargetAccountID,OriginAccountID,StatusID", AllowZero: true},
+			{Fields: "NotificationType,TargetAccountID,OriginAccountID,StatusOrEditID", AllowZero: true},
 		},
 		MaxSize:   cap,
 		IgnoreErr: ignoreErrors,
@@ -920,7 +1199,7 @@ func (c *Caches) initPoll() {
 		return p2
 	}
 
-	c.GTS.Poll.Init(structr.CacheConfig[*gtsmodel.Poll]{
+	c.DB.Poll.Init(structr.CacheConfig[*gtsmodel.Poll]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "StatusID"},
@@ -954,7 +1233,7 @@ func (c *Caches) initPollVote() {
 		return v2
 	}
 
-	c.GTS.PollVote.Init(structr.CacheConfig[*gtsmodel.PollVote]{
+	c.DB.PollVote.Init(structr.CacheConfig[*gtsmodel.PollVote]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "PollID", Multiple: true},
@@ -975,7 +1254,7 @@ func (c *Caches) initPollVoteIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.PollVoteIDs.Init(0, cap)
+	c.DB.PollVoteIDs.Init(0, cap)
 }
 
 func (c *Caches) initReport() {
@@ -1003,7 +1282,7 @@ func (c *Caches) initReport() {
 		return r2
 	}
 
-	c.GTS.Report.Init(structr.CacheConfig[*gtsmodel.Report]{
+	c.DB.Report.Init(structr.CacheConfig[*gtsmodel.Report]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 		},
@@ -1013,6 +1292,66 @@ func (c *Caches) initReport() {
 	})
 }
 
+func (c *Caches) initScheduledStatus() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofScheduledStatus(), // model in-mem size.
+		config.GetCacheScheduledStatusMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(s1 *gtsmodel.ScheduledStatus) *gtsmodel.ScheduledStatus {
+		s2 := new(gtsmodel.ScheduledStatus)
+		*s2 = *s1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		s2.Account = nil
+		s2.Application = nil
+		s2.MediaAttachments = nil
+
+		return s2
+	}
+
+	c.DB.ScheduledStatus.Init(structr.CacheConfig[*gtsmodel.ScheduledStatus]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "AccountID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateScheduledStatus,
+	})
+}
+
+func (c *Caches) initSinBinStatus() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofSinBinStatus(), // model in-mem size.
+		config.GetCacheSinBinStatusMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(s1 *gtsmodel.SinBinStatus) *gtsmodel.SinBinStatus {
+		s2 := new(gtsmodel.SinBinStatus)
+		*s2 = *s1
+		return s2
+	}
+
+	c.DB.SinBinStatus.Init(structr.CacheConfig[*gtsmodel.SinBinStatus]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "URI"},
+		},
+		MaxSize:   cap,
+		IgnoreErr: ignoreErrors,
+		Copy:      copyF,
+	})
+}
+
 func (c *Caches) initStatus() {
 	// Calculate maximum cache size.
 	cap := calculateResultCacheMax(
@@ -1040,11 +1379,12 @@ func (c *Caches) initStatus() {
 		s2.Mentions = nil
 		s2.Emojis = nil
 		s2.CreatedWithApplication = nil
+		s2.Edits = nil
 
 		return s2
 	}
 
-	c.GTS.Status.Init(structr.CacheConfig[*gtsmodel.Status]{
+	c.DB.Status.Init(structr.CacheConfig[*gtsmodel.Status]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -1060,6 +1400,86 @@ func (c *Caches) initStatus() {
 	})
 }
 
+func (c *Caches) initStatusBookmark() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofStatusBookmark(), // model in-mem size.
+		config.GetCacheStatusBookmarkMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(s1 *gtsmodel.StatusBookmark) *gtsmodel.StatusBookmark {
+		s2 := new(gtsmodel.StatusBookmark)
+		*s2 = *s1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		s2.Account = nil
+		s2.TargetAccount = nil
+		s2.Status = nil
+
+		return s2
+	}
+
+	c.DB.StatusBookmark.Init(structr.CacheConfig[*gtsmodel.StatusBookmark]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "AccountID,StatusID"},
+			{Fields: "AccountID", Multiple: true},
+			{Fields: "TargetAccountID", Multiple: true},
+			{Fields: "StatusID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateStatusBookmark,
+	})
+}
+
+func (c *Caches) initStatusBookmarkIDs() {
+	// Calculate maximum cache size.
+	cap := calculateSliceCacheMax(
+		config.GetCacheStatusBookmarkIDsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.StatusBookmarkIDs.Init(0, cap)
+}
+
+func (c *Caches) initStatusEdit() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofStatusEdit(), // model in-mem size.
+		config.GetCacheStatusEditMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(s1 *gtsmodel.StatusEdit) *gtsmodel.StatusEdit {
+		s2 := new(gtsmodel.StatusEdit)
+		*s2 = *s1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		s2.Attachments = nil
+
+		return s2
+	}
+
+	c.DB.StatusEdit.Init(structr.CacheConfig[*gtsmodel.StatusEdit]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "StatusID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateStatusEdit,
+	})
+}
+
 func (c *Caches) initStatusFave() {
 	// Calculate maximum cache size.
 	cap := calculateResultCacheMax(
@@ -1083,9 +1503,10 @@ func (c *Caches) initStatusFave() {
 		return f2
 	}
 
-	c.GTS.StatusFave.Init(structr.CacheConfig[*gtsmodel.StatusFave]{
+	c.DB.StatusFave.Init(structr.CacheConfig[*gtsmodel.StatusFave]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
+			{Fields: "URI"},
 			{Fields: "AccountID,StatusID"},
 			{Fields: "StatusID", Multiple: true},
 		},
@@ -1104,7 +1525,7 @@ func (c *Caches) initStatusFaveIDs() {
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.StatusFaveIDs.Init(0, cap)
+	c.DB.StatusFaveIDs.Init(0, cap)
 }
 
 func (c *Caches) initTag() {
@@ -1122,7 +1543,7 @@ func (c *Caches) initTag() {
 		return m2
 	}
 
-	c.GTS.Tag.Init(structr.CacheConfig[*gtsmodel.Tag]{
+	c.DB.Tag.Init(structr.CacheConfig[*gtsmodel.Tag]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "Name"},
@@ -1135,7 +1556,7 @@ func (c *Caches) initTag() {
 
 func (c *Caches) initThreadMute() {
 	cap := calculateResultCacheMax(
-		sizeOfThreadMute(), // model in-mem size.
+		sizeofThreadMute(), // model in-mem size.
 		config.GetCacheThreadMuteMemRatio(),
 	)
 
@@ -1147,16 +1568,47 @@ func (c *Caches) initThreadMute() {
 		return t2
 	}
 
-	c.GTS.ThreadMute.Init(structr.CacheConfig[*gtsmodel.ThreadMute]{
+	c.DB.ThreadMute.Init(structr.CacheConfig[*gtsmodel.ThreadMute]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "ThreadID", Multiple: true},
 			{Fields: "AccountID", Multiple: true},
 			{Fields: "ThreadID,AccountID"},
 		},
-		MaxSize:   cap,
-		IgnoreErr: ignoreErrors,
-		Copy:      copyF,
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateThreadMute,
+	})
+}
+
+func (c *Caches) initToken() {
+	// Calculate maximum cache size.
+	cap := calculateResultCacheMax(
+		sizeofToken(), // model in-mem size.
+		config.GetCacheTokenMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(t1 *gtsmodel.Token) *gtsmodel.Token {
+		t2 := new(gtsmodel.Token)
+		*t2 = *t1
+		return t2
+	}
+
+	c.DB.Token.Init(structr.CacheConfig[*gtsmodel.Token]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "Code"},
+			{Fields: "Access"},
+			{Fields: "Refresh"},
+			{Fields: "ClientID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateToken,
 	})
 }
 
@@ -1175,7 +1627,7 @@ func (c *Caches) initTombstone() {
 		return t2
 	}
 
-	c.GTS.Tombstone.Init(structr.CacheConfig[*gtsmodel.Tombstone]{
+	c.DB.Tombstone.Init(structr.CacheConfig[*gtsmodel.Tombstone]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "URI"},
@@ -1207,7 +1659,7 @@ func (c *Caches) initUser() {
 		return u2
 	}
 
-	c.GTS.User.Init(structr.CacheConfig[*gtsmodel.User]{
+	c.DB.User.Init(structr.CacheConfig[*gtsmodel.User]{
 		Indices: []structr.IndexConfig{
 			{Fields: "ID"},
 			{Fields: "AccountID"},
@@ -1222,19 +1674,84 @@ func (c *Caches) initUser() {
 	})
 }
 
-func (c *Caches) initWebfinger() {
-	// Calculate maximum cache size.
-	cap := calculateCacheMax(
-		sizeofURIStr, sizeofURIStr,
-		config.GetCacheWebfingerMemRatio(),
+func (c *Caches) initUserMute() {
+	cap := calculateResultCacheMax(
+		sizeofUserMute(), // model in-mem size.
+		config.GetCacheUserMuteMemRatio(),
 	)
 
 	log.Infof(nil, "cache size = %d", cap)
 
-	c.GTS.Webfinger = new(ttl.Cache[string, string])
-	c.GTS.Webfinger.Init(
-		0,
-		cap,
-		24*time.Hour,
+	copyF := func(u1 *gtsmodel.UserMute) *gtsmodel.UserMute {
+		u2 := new(gtsmodel.UserMute)
+		*u2 = *u1
+
+		// Don't include ptr fields that
+		// will be populated separately.
+		// See internal/db/bundb/relationship_mute.go.
+		u2.Account = nil
+		u2.TargetAccount = nil
+
+		return u2
+	}
+
+	c.DB.UserMute.Init(structr.CacheConfig[*gtsmodel.UserMute]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "AccountID,TargetAccountID"},
+			{Fields: "AccountID", Multiple: true},
+			{Fields: "TargetAccountID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Copy:       copyF,
+		Invalidate: c.OnInvalidateUserMute,
+	})
+}
+
+func (c *Caches) initUserMuteIDs() {
+	cap := calculateSliceCacheMax(
+		config.GetCacheUserMuteIDsMemRatio(),
 	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.UserMuteIDs.Init(0, cap)
+}
+
+func (c *Caches) initWebPushSubscription() {
+	cap := calculateResultCacheMax(
+		sizeofWebPushSubscription(), // model in-mem size.
+		config.GetCacheWebPushSubscriptionMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	copyF := func(s1 *gtsmodel.WebPushSubscription) *gtsmodel.WebPushSubscription {
+		s2 := new(gtsmodel.WebPushSubscription)
+		*s2 = *s1
+		return s2
+	}
+
+	c.DB.WebPushSubscription.Init(structr.CacheConfig[*gtsmodel.WebPushSubscription]{
+		Indices: []structr.IndexConfig{
+			{Fields: "ID"},
+			{Fields: "TokenID"},
+			{Fields: "AccountID", Multiple: true},
+		},
+		MaxSize:    cap,
+		IgnoreErr:  ignoreErrors,
+		Invalidate: c.OnInvalidateWebPushSubscription,
+		Copy:       copyF,
+	})
+}
+
+func (c *Caches) initWebPushSubscriptionIDs() {
+	cap := calculateSliceCacheMax(
+		config.GetCacheWebPushSubscriptionIDsMemRatio(),
+	)
+
+	log.Infof(nil, "cache size = %d", cap)
+
+	c.DB.WebPushSubscriptionIDs.Init(0, cap)
 }
