@@ -22,22 +22,20 @@ import (
 	"errors"
 	"time"
 
-	"github.com/superseriousbusiness/gotosocial/internal/db"
-	"github.com/superseriousbusiness/gotosocial/internal/gtscontext"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
-	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
-	"github.com/superseriousbusiness/gotosocial/internal/media"
-	"github.com/superseriousbusiness/gotosocial/internal/paging"
-	"github.com/superseriousbusiness/gotosocial/internal/regexes"
-	"github.com/superseriousbusiness/gotosocial/internal/uris"
+	"code.superseriousbusiness.org/gotosocial/internal/db"
+	"code.superseriousbusiness.org/gotosocial/internal/gtscontext"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/gotosocial/internal/gtsmodel"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
+	"code.superseriousbusiness.org/gotosocial/internal/media"
+	"code.superseriousbusiness.org/gotosocial/internal/paging"
+	"code.superseriousbusiness.org/gotosocial/internal/regexes"
+	"code.superseriousbusiness.org/gotosocial/internal/uris"
 )
 
 // Media encompasses a set of
 // media cleanup / admin utils.
-type Media struct {
-	*Cleaner
-}
+type Media struct{ *Cleaner }
 
 // All will execute all cleaner.Media utilities synchronously, including output logging.
 // Context will be checked for `gtscontext.DryRun()` in order to actually perform the action.
@@ -95,11 +93,12 @@ func (m *Media) LogFixCacheStates(ctx context.Context) {
 func (m *Media) PruneOrphaned(ctx context.Context) (int, error) {
 	var files []string
 
-	// All media files in storage will have path fitting: {$account}/{$type}/{$size}/{$id}.{$ext}
-	if err := m.state.Storage.WalkKeys(ctx, func(ctx context.Context, path string) error {
-		// Check for our expected fileserver path format.
+	// All media in storage will have path: {$account}/{$type}/{$size}/{$id}.{$ext}
+	if err := m.state.Storage.WalkKeys(ctx, func(path string) error {
+
+		// Check for expected fileserver path format.
 		if !regexes.FilePath.MatchString(path) {
-			log.Warn(ctx, "unexpected storage item: %s", path)
+			log.Warnf(ctx, "unexpected storage item: %s", path)
 			return nil
 		}
 
@@ -376,6 +375,25 @@ func (m *Media) pruneUnused(ctx context.Context, media *gtsmodel.MediaAttachment
 		}
 	}
 
+	// Check whether we have the required scheduled status for media.
+	scheduledStatus, missing, err := m.getRelatedScheduledStatus(ctx, media)
+	if err != nil {
+		return false, err
+	} else if missing {
+		l.Debug("deleting due to missing scheduled status")
+		return true, m.delete(ctx, media)
+	}
+
+	if scheduledStatus != nil {
+		// Check whether still attached to status.
+		for _, id := range scheduledStatus.MediaIDs {
+			if id == media.ID {
+				l.Debug("skippping as attached to scheduled status")
+				return false, nil
+			}
+		}
+	}
+
 	// Media totally unused, delete it.
 	l.Debug("deleting unused media")
 	return true, m.delete(ctx, media)
@@ -475,9 +493,21 @@ func (m *Media) uncacheRemote(ctx context.Context, after time.Time, media *gtsmo
 			return false, nil
 		}
 
-		if status != nil && status.FetchedAt.After(after) {
-			l.Debug("skipping due to recently fetched status")
-			return false, nil
+		if status != nil {
+			// Check if recently used status.
+			if status.FetchedAt.After(after) {
+				l.Debug("skipping due to recently fetched status")
+				return false, nil
+			}
+
+			// Check whether status is bookmarked by active accounts.
+			bookmarked, err := m.state.DB.IsStatusBookmarked(ctx, status.ID)
+			if err != nil {
+				return false, err
+			} else if bookmarked {
+				l.Debug("skipping due to bookmarked status")
+				return false, nil
+			}
 		}
 	}
 
@@ -532,6 +562,29 @@ func (m *Media) getRelatedStatus(ctx context.Context, media *gtsmodel.MediaAttac
 	return status, false, nil
 }
 
+func (m *Media) getRelatedScheduledStatus(ctx context.Context, media *gtsmodel.MediaAttachment) (*gtsmodel.ScheduledStatus, bool, error) {
+	if media.ScheduledStatusID == "" {
+		// no related status.
+		return nil, false, nil
+	}
+
+	// Load the status related to this media.
+	status, err := m.state.DB.GetScheduledStatusByID(
+		gtscontext.SetBarebones(ctx),
+		media.ScheduledStatusID,
+	)
+	if err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return nil, false, gtserror.Newf("error fetching scheduled status by id %s: %w", media.ScheduledStatusID, err)
+	}
+
+	if status == nil {
+		// status is missing.
+		return nil, true, nil
+	}
+
+	return status, false, nil
+}
+
 func (m *Media) uncache(ctx context.Context, media *gtsmodel.MediaAttachment) error {
 	if gtscontext.DryRun(ctx) {
 		// Dry run, do nothing.
