@@ -24,18 +24,18 @@ import (
 	"net/url"
 	"strings"
 
-	"codeberg.org/gruf/go-kv"
-	"github.com/superseriousbusiness/activity/pub"
-	"github.com/superseriousbusiness/activity/streams"
-	"github.com/superseriousbusiness/activity/streams/vocab"
-	"github.com/superseriousbusiness/gotosocial/internal/ap"
-	"github.com/superseriousbusiness/gotosocial/internal/config"
-	"github.com/superseriousbusiness/gotosocial/internal/db"
-	"github.com/superseriousbusiness/gotosocial/internal/gtscontext"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
-	"github.com/superseriousbusiness/gotosocial/internal/uris"
-	"github.com/superseriousbusiness/gotosocial/internal/util"
+	"code.superseriousbusiness.org/activity/pub"
+	"code.superseriousbusiness.org/activity/streams"
+	"code.superseriousbusiness.org/activity/streams/vocab"
+	"code.superseriousbusiness.org/gotosocial/internal/ap"
+	"code.superseriousbusiness.org/gotosocial/internal/config"
+	"code.superseriousbusiness.org/gotosocial/internal/db"
+	"code.superseriousbusiness.org/gotosocial/internal/gtscontext"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
+	"code.superseriousbusiness.org/gotosocial/internal/uris"
+	"code.superseriousbusiness.org/gotosocial/internal/util/xslices"
+	"codeberg.org/gruf/go-kv/v2"
 )
 
 type errOtherIRIBlocked struct {
@@ -44,7 +44,7 @@ type errOtherIRIBlocked struct {
 	iriStrs     []string
 }
 
-func (e errOtherIRIBlocked) Error() string {
+func (e *errOtherIRIBlocked) Error() string {
 	iriStrsNice := "[" + strings.Join(e.iriStrs, ", ") + "]"
 	if e.domainBlock {
 		return "domain block exists for one or more of " + iriStrsNice
@@ -67,7 +67,7 @@ func newErrOtherIRIBlocked(
 		e.iriStrs = append(e.iriStrs, iri.String())
 	}
 
-	return e
+	return &e
 }
 
 /*
@@ -116,33 +116,36 @@ func (f *Federator) PostInboxRequestBodyHook(ctx context.Context, r *http.Reques
 		otherIRIs = append(otherIRIs, ap.ExtractCcURIs(addressable)...)
 	}
 
-	// Now perform the same checks, but for the Object(s) of the Activity.
+	// Now perform the same checks, but
+	// for any Object(s) of the Activity.
 	objectProp := activity.GetActivityStreamsObject()
-	for iter := objectProp.Begin(); iter != objectProp.End(); iter = iter.Next() {
-		if iter.IsIRI() {
-			otherIRIs = append(otherIRIs, iter.GetIRI())
-			continue
-		}
-
-		t := iter.GetType()
-		if t == nil {
-			continue
-		}
-
-		objectID, err := pub.GetId(t)
-		if err == nil {
-			otherIRIs = append(otherIRIs, objectID)
-		}
-
-		if replyToable, ok := t.(ap.ReplyToable); ok {
-			if inReplyToURI := ap.ExtractInReplyToURI(replyToable); inReplyToURI != nil {
-				otherIRIs = append(otherIRIs, inReplyToURI)
+	if objectProp != nil {
+		for iter := objectProp.Begin(); iter != objectProp.End(); iter = iter.Next() {
+			if iter.IsIRI() {
+				otherIRIs = append(otherIRIs, iter.GetIRI())
+				continue
 			}
-		}
 
-		if addressable, ok := t.(ap.Addressable); ok {
-			otherIRIs = append(otherIRIs, ap.ExtractToURIs(addressable)...)
-			otherIRIs = append(otherIRIs, ap.ExtractCcURIs(addressable)...)
+			t := iter.GetType()
+			if t == nil {
+				continue
+			}
+
+			objectID, err := pub.GetId(t)
+			if err == nil {
+				otherIRIs = append(otherIRIs, objectID)
+			}
+
+			if replyToable, ok := t.(ap.ReplyToable); ok {
+				if inReplyToURI := ap.ExtractInReplyToURI(replyToable); inReplyToURI != nil {
+					otherIRIs = append(otherIRIs, inReplyToURI)
+				}
+			}
+
+			if addressable, ok := t.(ap.Addressable); ok {
+				otherIRIs = append(otherIRIs, ap.ExtractToURIs(addressable)...)
+				otherIRIs = append(otherIRIs, ap.ExtractCcURIs(addressable)...)
+			}
 		}
 	}
 
@@ -162,7 +165,9 @@ func (f *Federator) PostInboxRequestBodyHook(ctx context.Context, r *http.Reques
 
 	// OtherIRIs will likely contain some
 	// duplicate entries now, so remove them.
-	otherIRIs = util.UniqueURIs(otherIRIs)
+	otherIRIs = xslices.DeduplicateFunc(otherIRIs,
+		(*url.URL).String, // serialized URL is 'key()'
+	)
 
 	// Finished, set other IRIs on the context
 	// so they can be checked for blocks later.
@@ -454,39 +459,8 @@ func (f *Federator) FederatingCallbacks(ctx context.Context) (
 	other []any,
 	err error,
 ) {
-	wrapped = pub.FederatingWrappedCallbacks{
-		// OnFollow determines what action to take for this
-		// particular callback if a Follow Activity is handled.
-		//
-		// For our implementation, we always want to do nothing
-		// because we have internal logic for handling follows.
-		OnFollow: pub.OnFollowDoNothing,
-	}
-
-	// Override some default behaviors to trigger our own side effects.
-	other = []any{
-		func(ctx context.Context, undo vocab.ActivityStreamsUndo) error {
-			return f.FederatingDB().Undo(ctx, undo)
-		},
-		func(ctx context.Context, accept vocab.ActivityStreamsAccept) error {
-			return f.FederatingDB().Accept(ctx, accept)
-		},
-		func(ctx context.Context, reject vocab.ActivityStreamsReject) error {
-			return f.FederatingDB().Reject(ctx, reject)
-		},
-		func(ctx context.Context, announce vocab.ActivityStreamsAnnounce) error {
-			return f.FederatingDB().Announce(ctx, announce)
-		},
-	}
-
-	// Define some of our own behaviors which are not
-	// overrides of the default pub.FederatingWrappedCallbacks.
-	other = append(other, []any{
-		func(ctx context.Context, move vocab.ActivityStreamsMove) error {
-			return f.FederatingDB().Move(ctx, move)
-		},
-	}...)
-
+	wrapped = f.wrapped
+	other = f.callback
 	return
 }
 
