@@ -25,16 +25,16 @@ import (
 	"net/url"
 	"strings"
 
-	"codeberg.org/gruf/go-kv"
-	apimodel "github.com/superseriousbusiness/gotosocial/internal/api/model"
-	"github.com/superseriousbusiness/gotosocial/internal/config"
-	"github.com/superseriousbusiness/gotosocial/internal/db"
-	"github.com/superseriousbusiness/gotosocial/internal/gtscontext"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
-	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
-	"github.com/superseriousbusiness/gotosocial/internal/text"
-	"github.com/superseriousbusiness/gotosocial/internal/util"
+	apimodel "code.superseriousbusiness.org/gotosocial/internal/api/model"
+	"code.superseriousbusiness.org/gotosocial/internal/config"
+	"code.superseriousbusiness.org/gotosocial/internal/db"
+	"code.superseriousbusiness.org/gotosocial/internal/gtscontext"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/gotosocial/internal/gtsmodel"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
+	"code.superseriousbusiness.org/gotosocial/internal/text"
+	"code.superseriousbusiness.org/gotosocial/internal/util"
+	"codeberg.org/gruf/go-kv/v2"
 )
 
 const (
@@ -62,14 +62,15 @@ func (p *Processor) Get(
 	req *apimodel.SearchRequest,
 ) (*apimodel.SearchResult, gtserror.WithCode) {
 	var (
-		maxID     = req.MaxID
-		minID     = req.MinID
-		limit     = req.Limit
-		offset    = req.Offset
-		query     = strings.TrimSpace(req.Query)                      // Trim trailing/leading whitespace.
-		queryType = strings.TrimSpace(strings.ToLower(req.QueryType)) // Trim trailing/leading whitespace; convert to lowercase.
-		resolve   = req.Resolve
-		following = req.Following
+		maxID         = req.MaxID
+		minID         = req.MinID
+		limit         = req.Limit
+		offset        = req.Offset
+		query         = strings.TrimSpace(req.Query)                      // Trim trailing/leading whitespace.
+		queryType     = strings.TrimSpace(strings.ToLower(req.QueryType)) // Trim trailing/leading whitespace; convert to lowercase.
+		resolve       = req.Resolve
+		following     = req.Following
+		fromAccountID = req.AccountID
 
 		// Include instance accounts in the first
 		// parts of this search. This will be
@@ -114,6 +115,7 @@ func (p *Processor) Get(
 			{"queryType", queryType},
 			{"resolve", resolve},
 			{"following", following},
+			{"fromAccountID", fromAccountID},
 		}...).
 		Debugf("beginning search")
 
@@ -309,6 +311,7 @@ func (p *Processor) Get(
 		query,
 		queryType,
 		following,
+		fromAccountID,
 		appendAccount,
 		appendStatus,
 	); err != nil && !errors.Is(err, db.ErrNoEntries) {
@@ -487,7 +490,7 @@ func (p *Processor) byURI(
 
 	if includeAccounts(queryType) {
 		// Check if URI points to an account.
-		foundAccount, err := p.accountByURI(ctx, requestingAccount, uri, resolve)
+		foundAccounts, err := p.accountsByURI(ctx, requestingAccount, uri, resolve)
 		if err != nil {
 			// Check for semi-expected error types.
 			// On one of these, we can continue.
@@ -505,7 +508,9 @@ func (p *Processor) byURI(
 		} else {
 			// Hit! Return early since it's extremely unlikely
 			// a status and an account will have the same URL.
-			appendAccount(foundAccount)
+			for _, foundAccount := range foundAccounts {
+				appendAccount(foundAccount)
+			}
 			return nil
 		}
 	}
@@ -519,7 +524,7 @@ func (p *Processor) byURI(
 			switch {
 			case gtserror.IsUnretrievable(err),
 				gtserror.IsWrongType(err),
-				gtserror.NotPermitted(err):
+				gtserror.IsNotPermitted(err):
 				log.Debugf(ctx,
 					"semi-expected error type looking up %s as status: %v",
 					uri, err,
@@ -541,35 +546,42 @@ func (p *Processor) byURI(
 	return nil
 }
 
-// accountByURI looks for one account with the given URI.
+// accountsByURI looks for one account with the given URI/ID,
+// then if nothing is found, multiple accounts with the given URL.
+//
 // If resolve is false, it will only look in the database.
 // If resolve is true, it will try to resolve the account
 // from remote using the URI, if necessary.
 //
 // Will return either a hit, ErrNotRetrievable, ErrWrongType,
 // or a real error that the caller should handle.
-func (p *Processor) accountByURI(
+func (p *Processor) accountsByURI(
 	ctx context.Context,
 	requestingAccount *gtsmodel.Account,
 	uri *url.URL,
 	resolve bool,
-) (*gtsmodel.Account, error) {
+) ([]*gtsmodel.Account, error) {
 	if resolve {
 		// We're allowed to resolve, leave the
 		// rest up to the dereferencer functions.
+		//
+		// Allow dereferencing by URL and not just URI;
+		// there are many cases where someone might
+		// paste a URL into the search bar.
 		account, _, err := p.federator.GetAccountByURI(
 			gtscontext.SetFastFail(ctx),
 			requestingAccount.Username,
 			uri,
+			true,
 		)
 
-		return account, err
+		return []*gtsmodel.Account{account}, err
 	}
 
 	// We're not allowed to resolve; search database only.
 	uriStr := uri.String() // stringify uri just once
 
-	// Search by ActivityPub URI.
+	// Search for single acct by ActivityPub URI.
 	account, err := p.state.DB.GetAccountByURI(ctx, uriStr)
 	if err != nil && !errors.Is(err, db.ErrNoEntries) {
 		err = gtserror.Newf("error checking database for account using URI %s: %w", uriStr, err)
@@ -578,22 +590,22 @@ func (p *Processor) accountByURI(
 
 	if account != nil {
 		// We got a hit! No need to continue.
-		return account, nil
+		return []*gtsmodel.Account{account}, nil
 	}
 
-	// No hit yet. Fallback to try by URL.
-	account, err = p.state.DB.GetAccountByURL(ctx, uriStr)
+	// No hit yet. Fallback to look for any accounts with URL.
+	accounts, err := p.state.DB.GetAccountsByURL(ctx, uriStr)
 	if err != nil && !errors.Is(err, db.ErrNoEntries) {
-		err = gtserror.Newf("error checking database for account using URL %s: %w", uriStr, err)
+		err = gtserror.Newf("error checking database for accounts using URL %s: %w", uriStr, err)
 		return nil, err
 	}
 
-	if account != nil {
-		// We got a hit! No need to continue.
-		return account, nil
+	if len(accounts) != 0 {
+		// We got hits! No need to continue.
+		return accounts, nil
 	}
 
-	err = fmt.Errorf("account %s could not be retrieved locally and we cannot resolve", uriStr)
+	err = fmt.Errorf("account(s) %s could not be retrieved locally and we cannot resolve", uriStr)
 	return nil, gtserror.SetUnretrievable(err)
 }
 
@@ -743,6 +755,7 @@ func (p *Processor) byText(
 	query string,
 	queryType string,
 	following bool,
+	fromAccountID string,
 	appendAccount func(*gtsmodel.Account),
 	appendStatus func(*gtsmodel.Status),
 ) error {
@@ -779,6 +792,7 @@ func (p *Processor) byText(
 			limit,
 			offset,
 			query,
+			fromAccountID,
 			appendStatus,
 		); err != nil {
 			return err
@@ -826,12 +840,30 @@ func (p *Processor) statusesByText(
 	limit int,
 	offset int,
 	query string,
+	fromAccountID string,
 	appendStatus func(*gtsmodel.Status),
 ) error {
+	parsed, err := p.parseQuery(ctx, query)
+	if err != nil {
+		return err
+	}
+	query = parsed.query
+	// If the owning account for statuses was not provided as the account_id query parameter,
+	// it may still have been provided as a search operator in the query string.
+	if fromAccountID == "" {
+		fromAccountID = parsed.fromAccountID
+	}
+
 	statuses, err := p.state.DB.SearchForStatuses(
 		ctx,
 		requestingAccountID,
-		query, maxID, minID, limit, offset)
+		query,
+		fromAccountID,
+		maxID,
+		minID,
+		limit,
+		offset,
+	)
 	if err != nil && !errors.Is(err, db.ErrNoEntries) {
 		return gtserror.Newf("error checking database for statuses using text %s: %w", query, err)
 	}
@@ -842,3 +874,60 @@ func (p *Processor) statusesByText(
 
 	return nil
 }
+
+// parsedQuery represents the results of parsing the search operator terms within a query.
+type parsedQuery struct {
+	// query is the original search query text with operator terms removed.
+	query string
+	// fromAccountID is the account from a successfully resolved `from:` operator, if present.
+	fromAccountID string
+}
+
+// parseQuery parses query text and handles any search operator terms present.
+func (p *Processor) parseQuery(ctx context.Context, query string) (parsed parsedQuery, err error) {
+	queryPartSeparator := " "
+	queryParts := strings.Split(query, queryPartSeparator)
+	nonOperatorQueryParts := make([]string, 0, len(queryParts))
+	for _, queryPart := range queryParts {
+		if arg, hasPrefix := strings.CutPrefix(queryPart, "from:"); hasPrefix {
+			parsed.fromAccountID, err = p.parseFromOperatorArg(ctx, arg)
+			if err != nil {
+				return
+			}
+		} else {
+			nonOperatorQueryParts = append(nonOperatorQueryParts, queryPart)
+		}
+	}
+	parsed.query = strings.Join(nonOperatorQueryParts, queryPartSeparator)
+	return
+}
+
+// parseFromOperatorArg attempts to parse the from: operator's argument as an account name,
+// and returns the account ID if possible. Allows specifying an account name with or without a leading @.
+func (p *Processor) parseFromOperatorArg(ctx context.Context, namestring string) (string, error) {
+	if namestring == "" {
+		return "", gtserror.New(
+			"the 'from:' search operator requires an account name, but it wasn't provided",
+		)
+	}
+	if namestring[0] != '@' {
+		namestring = "@" + namestring
+	}
+
+	username, domain, err := util.ExtractNamestringParts(namestring)
+	if err != nil {
+		return "", gtserror.Newf(
+			"the 'from:' search operator couldn't parse its argument as an account name: %w",
+			err,
+		)
+	}
+	account, err := p.state.DB.GetAccountByUsernameDomain(gtscontext.SetBarebones(ctx), username, domain)
+	if err != nil {
+		return "", gtserror.Newf(
+			"the 'from:' search operator couldn't find the requested account name: %w",
+			err,
+		)
+	}
+
+	return account.ID, nil
+}
